"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const project_1 = require("../../utils/project");
const workspace_models_1 = require("@schematics/angular/utility/workspace-models");
const paths_1 = require("../../utils/paths");
const workspace_1 = require("@schematics/angular/utility/workspace");
function migration(options) {
    return schematics_1.chain([
        configure(options),
        updateFiles(options),
    ]);
}
exports.default = migration;
function updateAppInWorkspaceFile(options, workspace, project, name) {
    const projectRoot = core_1.normalize(project.root);
    const projectRootParts = projectRoot.split('/');
    const sourceRoot = project_1.getSourceRoot(project.sourceRoot);
    const sourceRootParts = sourceRoot.split('/');
    workspace.projects.delete(name);
    workspace.projects.add({
        name,
        root: projectRoot,
        sourceRoot,
        projectType: 'application',
        targets: {
            build: {
                builder: workspace_models_1.Builders.Browser,
                options: {
                    outputPath: paths_1.constructPath(['dist', 'playground']),
                    index: paths_1.constructPath([...sourceRootParts, 'index.html']),
                    main: paths_1.constructPath([...sourceRootParts, 'main.playground.ts']),
                    polyfills: paths_1.constructPath([...sourceRootParts, 'polyfills.ts']),
                    tsConfig: paths_1.constructPath([...projectRootParts, `tsconfig.playground.json`]),
                    aot: false,
                    assets: [
                        paths_1.constructPath([...sourceRootParts, 'favicon.ico']),
                        paths_1.constructPath([...sourceRootParts, 'assets']),
                    ],
                    styles: [
                        paths_1.constructPath([...sourceRootParts, `styles.${options.stylesExtension}`]),
                    ],
                    scripts: [],
                },
                configurations: {
                    production: {
                        fileReplacements: [
                            {
                                replace: paths_1.constructPath([...sourceRootParts, 'environments', 'environment.ts']),
                                with: paths_1.constructPath([...sourceRootParts, 'environments', 'environment.prod.ts']),
                            },
                        ],
                        optimization: true,
                        outputHashing: 'all',
                        sourceMap: false,
                        extractCss: true,
                        namedChunks: false,
                        extractLicenses: true,
                        vendorChunk: false,
                        buildOptimizer: true,
                    },
                },
            },
            serve: {
                builder: workspace_models_1.Builders.DevServer,
                options: {
                    browserTarget: 'playground:build',
                    port: 4201
                },
            }
        }
    });
    return workspace_1.updateWorkspace(workspace);
}
function configure(options) {
    return (tree) => __awaiter(this, void 0, void 0, function* () {
        const workspace = yield workspace_1.getWorkspace(tree);
        const project = project_1.getProject(workspace, options, 'application');
        if (!project) {
            throw new schematics_1.SchematicsException('Your app must have at least 1 project to use Playground.');
        }
        let stylesExtension = 'css';
        const buildTarget = project.targets.get('build');
        if (buildTarget
            && buildTarget.options
            && buildTarget.options.styles
            && core_1.isJsonArray(buildTarget.options.styles)) {
            const mainStyle = buildTarget.options.styles
                .find((path) => typeof path === 'string'
                ? path.includes('/styles.')
                : path.input.includes('/styles.'));
            if (mainStyle) {
                // @ts-ignore
                const mainStyleString = typeof mainStyle === 'string' ? mainStyle : mainStyle.input;
                stylesExtension = mainStyleString.split('.').pop();
            }
        }
        return schematics_1.chain([
            updateAppInWorkspaceFile({ stylesExtension }, workspace, project, 'playground'),
        ]);
    });
}
function updateFiles(options) {
    return (tree) => __awaiter(this, void 0, void 0, function* () {
        const workspace = yield workspace_1.getWorkspace(tree);
        const project = project_1.getProject(workspace, options);
        const sourceRoot = project_1.getSourceRoot(project === null || project === void 0 ? void 0 : project.sourceRoot);
        const tsconfigJsonTemplateSource = schematics_1.apply(schematics_1.url('./files'), [
            schematics_1.filter(path => path.endsWith('tsconfig.playground.json')),
            schematics_1.template({}),
            overwriteIfExists(tree),
        ]);
        const playgroundMainTemplateSource = schematics_1.apply(schematics_1.url('./files'), [
            schematics_1.filter(path => path.endsWith('main.playground.ts')),
            schematics_1.template({}),
            schematics_1.move(sourceRoot),
            overwriteIfExists(tree),
        ]);
        return schematics_1.chain([
            schematics_1.mergeWith(tsconfigJsonTemplateSource),
            schematics_1.mergeWith(playgroundMainTemplateSource),
        ]);
    });
}
// Add to resolve issue with mergeWith not working when using MergeStrategy.Overwrite
// When not using this function I get this error:  "Error [file] already exists."
// https://github.com/angular/angular-cli/issues/11337#issuecomment-516543220
function overwriteIfExists(tree) {
    return schematics_1.forEach(fileEntry => {
        if (tree.exists(fileEntry.path)) {
            tree.overwrite(fileEntry.path, fileEntry.content);
            return null;
        }
        return fileEntry;
    });
}
//# sourceMappingURL=index.js.map